/*globals define, Image*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/shop/ui/dashboard-notifications-test',
    'testsuite/sinon',
    'jquery',
    'de_epages/shop/ui/dashboard-notifications',
    "util/string",

    '$ready!'
], function (QUnit, fixture, sinon, $, de_epages, str) {

    // set module wide fixture property
    QUnit.fixtureVar = fixture;

    QUnit.module("de_epages/product/ui/dashboard-notifications", {

        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("widget test presence of methods", function () {

        $.each([
            '_create',
            'destroy'
        ], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.shopUiDashboardNotifications.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("de_epages/shop/ui/dashboard-notifications - Test basic widget functionality", function () {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $dashboardNotificationContainer = de_epages('.ep-dashboard-notification-container'),
            selectors = {
                hiddenRowNumber: '#ep-dashboard-hidden-notification-number',
                showMoreButton:'#ep-dashboard-show-more-button',
                showMoreRow: '.ep-dashboard-show-more',
                emptyNotificationRow: '.ep-dashboard-notification-empty',
                notificationRow: '.ep-dashboard-notification',
                removeButton: '.ep-dashboard-cancel'
            },

            $firstCancelElem = $('.ep-dashboard-cancel').eq(0),
            $dashboardTextShrinkTest,
            initSpy,
            destroySpy;

        // register spies on widget methods => number of executions can be investigated
        initSpy = sinon.spy($.ui.shopUiDashboardNotifications.prototype, '_create');
        destroySpy = sinon.spy($.ui.shopUiDashboardNotifications.prototype, 'destroy');

        QUnit.equal(
            initSpy.callCount,
            0,
            '*_create* method to initialize widget was not called until yet.'
        );

        $dashboardNotificationContainer.shopUiDashboardNotifications({
          showMore: true,
          selectors: selectors,
          removeChangeAction: "testChangeAction"
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            QUnit.equal(
                !!$dashboardNotificationContainer.shopUiDashboardNotifications('instance'),
                true,
                'shopUiDashboardNotifications widget instance exists.'
            );

            QUnit.equal(
                initSpy.callCount,
                1,
                '*_create* method to initialize widget was called once.'
            );

            QUnit.ok(
                $._data($firstCancelElem[0], 'events') && $._data($firstCancelElem[0], 'events').click,
                "Click event successfully registered for first cancel button."
            );

            $dashboardTextShrinkTest = $('.ep-dashboard-text-shrink-test');
            QUnit.equal(
                $dashboardTextShrinkTest.find('.ep-notification-text-shrinked').length,
                1,
                "Node for shrinked content is present."
            );
            QUnit.equal(
                $dashboardTextShrinkTest.find('.ep-notification-text-shrinked').text().startsWith(
                    str.shrink(
                        $dashboardTextShrinkTest.find('.ep-notification-text-original').text(),
                        {
                            length: 50,
                            spacer: ' ... ',
                            cut: false
                        }
                    )
                ),
                true,
                "Node for shrinked content contains the correct shrinked text."
            );

            $dashboardNotificationContainer.shopUiDashboardNotifications('destroy');

            window.setTimeout(function() {
                QUnit.equal(
                    destroySpy.callCount,
                    1,
                    '*destroy* method to initialize widget was called once.'
                );

                QUnit.equal(
                    !!($._data($firstCancelElem[0], 'events') && $._data($firstCancelElem[0], 'events').click),
                    false,
                    "Click event was successfully removed from the first cancel button by *destroy*."
                );

                QUnit.equal(
                    $dashboardTextShrinkTest.find('.ep-notification-text-shrinked').length,
                    0,
                    "Node for shrinked content was deleted by *destroy*."
                );

                QUnit.start();
            }, 100);
        }, 300);
    });

    QUnit.asyncTest("checking widget functionality (deleting single items)", function () {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var selectors = {
                hiddenRowNumber: '#ep-dashboard-hidden-notification-number',
                showMoreButton:'#ep-dashboard-show-more-button',
                showMoreRow: '.ep-dashboard-show-more',
                emptyNotificationRow: '.ep-dashboard-notification-empty',
                notificationRow: '.ep-dashboard-notification',
                removeButton: '.ep-dashboard-cancel'
            },
            currentId;

        ep.ajax = function(data) {
            QUnit.equal(
                data.data.ChangeAction,
                'testChangeAction',
                'Ajax: Correct ChangeAction used.'
            );

            QUnit.equal(
                data.data.ObjectID,
                currentId,
                'Ajax: Correct ObjectId used.'
            );

            return {
                done: function(fn) {
                    fn();

                    return {
                        fail: function() {
                            return;
                        }
                    };
                }
            };
        };

        de_epages('.ep-dashboard-notification-container').shopUiDashboardNotifications({
          showMore: true,
          selectors: selectors,
          removeChangeAction: "testChangeAction"
        });


        window.setTimeout(function () {
            var widget = ep('.ep-dashboard-notification-container').data('uiShopUiDashboardNotifications'),
                deleteableRows = ep(selectors.notificationRow + ':has(' + selectors.removeButton + ')');

            QUnit.equal(
                (widget !== undefined),
                true,
                'widget is resolved'
            );

            QUnit.equal(
                ep(selectors.notificationRow + ':not(.HideElement)').length,
                4,
                'four notifications are visible'
            );

            currentId = deleteableRows.eq(0).data('notificationConfig').id;
            deleteableRows.eq(0).find(selectors.removeButton).trigger('click');

            window.setTimeout(function() {

                QUnit.equal(
                    ep(selectors.notificationRow + ':not(.HideElement)').length,
                    4,
                    'four notifications are visible after deleting one'
                );

                QUnit.ok(
                    deleteableRows.eq(0).hasClass('HideElement'),
                    'the deleted notification is hidden'
                );

                QUnit.equal(
                    parseInt(ep(selectors.hiddenRowNumber).text()),
                    2,
                    'the hidden notifications count is two'
                );

                currentId = deleteableRows.eq(1).data('notificationConfig').id;
                deleteableRows.eq(1).find(selectors.removeButton).trigger('click');

                window.setTimeout(function() {

                    QUnit.equal(
                        ep(selectors.notificationRow + ':not(.HideElement)').length,
                        4,
                        'four notifications are visible'
                    );

                    QUnit.ok(
                        deleteableRows.eq(1).hasClass('HideElement'),
                        'the deleted notification is hidden'
                    );

                    QUnit.equal(
                        ep(selectors.showMoreRow).length,
                        1,
                        'show more row is still visible'
                    );

                    currentId = deleteableRows.eq(2).data('notificationConfig').id;
                    deleteableRows.eq(2).find(selectors.removeButton).trigger('click');

                    window.setTimeout(function() {

                        QUnit.equal(
                            ep(selectors.notificationRow + ':not(.HideElement)').length,
                            4,
                            'four notifications are visible'
                        );

                        QUnit.ok(
                            deleteableRows.eq(2).hasClass('HideElement'),
                            'the deleted notification is hidden'
                        );

                        QUnit.equal(
                            ep(selectors.showMoreRow).length,
                            0,
                            'show more row is hidden'
                        );


                        fixture.destroy();
                        QUnit.start();
                    }, 100);
                }, 100);
            }, 100);
        }, 100);
    });

    QUnit.asyncTest("checking widget functionality (Show All)", function () {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var selectors = {
                hiddenRowNumber: '#ep-dashboard-hidden-notification-number',
                showMoreButton:'#ep-dashboard-show-more-button',
                showMoreRow: '.ep-dashboard-show-more',
                notificationRow: '.ep-dashboard-notification',
                emptyNotificationRow: '.ep-dashboard-notification-empty',
                removeButton: '.ep-dashboard-cancel'
            };

        de_epages('.ep-dashboard-notification-container').shopUiDashboardNotifications({
          showMore: true,
          selectors: selectors,
          removeChangeAction: "testChangeAction"
        });

        QUnit.equal(
            ep(selectors.notificationRow + ':not(.HideElement)').length,
            4,
            'four notifications are visible'
        );

        ep(selectors.showMoreButton).trigger('click');

        QUnit.equal(
            ep(selectors.notificationRow + ':not(.HideElement)').length,
            7,
            'Seven notifications are visible after clicking show all'
        );

        fixture.destroy();
        QUnit.start();
    });

    QUnit.asyncTest("checking empty list functionality", function () {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();


        ep.ajax = function(data) {
            return {
                done: function(fn) {
                    fn();

                    return {
                        fail: function() {
                            return;
                        }
                    };
                }
            };
        };

        var selectors = {
                hiddenRowNumber: '#ep-dashboard-hidden-notification-number',
                showMoreButton:'#ep-dashboard-show-more-button',
                showMoreRow: '.ep-dashboard-show-more',
                notificationRow: '.ep-dashboard-notification',
                emptyNotificationRow: '.ep-dashboard-notification-empty',
                removeButton: '.ep-dashboard-cancel'
            },
            widgetElem = de_epages('.ep-dashboard-notification-container'),
            deleteButtons = ep(selectors.notificationRow + ' ' + selectors.removeButton);

        widgetElem.find(selectors.notificationRow).slice(1).remove();

        widgetElem.shopUiDashboardNotifications({
          showMore: true,
          selectors: selectors,
          removeChangeAction: "testChangeAction"
        });

        QUnit.equal(
            ep(selectors.notificationRow + ':not(.HideElement)').length,
            1,
            'one notification is visible'
        );

        deleteButtons.eq(0).trigger('click');

        QUnit.ok(
            !ep(selectors.emptyNotificationRow).hasClass('HideElement'),
            'empty notification is visible'
        );

        QUnit.ok(
            ep(selectors.notificationRow).hasClass('HideElement'),
            'notification are not visible'
        );

        fixture.destroy();
        QUnit.start();
    });
});
