/*global define window*/
/*jslint browser:true*/
define([
     'testsuite/qunit',
     'testsuite/qunit-fixture!de_epages/product/ui/variationattribute-unity-test.html',
     'jquery',
     'de_epages/product/ui/variationattribute-unity',
     'ep',

     'jquery/metaparse',
     '$ready!'
], function (QUnit, fixture, $, de_epages, ep) {
        'use strict';

    // overwrite default timeout
    window.timeoutOverwrite = 60;

    var bus = $('div'),
        options = {
            input: '.ep-variationassistant-attributevalueinput',
            initialInput: '.ep-variationassistant-attributevalueinput-initial',
            list: '.ep-variationassistant-attributelist',
            bus: $('.ep-variationassistant-attributelist').closest('form'),
            maxValues: 10
        },
        values;

    QUnit.module("de_epages/product/ui/variationattribute-unity", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("widget test presence of methods", function () {
        $.each([
            '_create',
            '_toggleAll',
            '_deleteRow',
            '_validate',
            '_createNewRow',
            'removeEmptyWarning',
            'getSelectedValues',
            '_validateSelectedValueCount',
            'isValid',
            'isEmpty',
            '_delay',
            'destroy'
        ], function (i, fn) {
               QUnit.equal(
                $.isFunction($.ui.productUiVariationattributeUnity.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("test validation with prefilled options", function () {
        fixture.init();
        var test = $('#test2'),
            input = test.find('.ep-variationassistant-attributevalueinput'),
            list = $('#test2 .ep-variationassistant-attributelist');

        $('.ep-js').metaparse();

        test.productUiVariationattributeUnity(options);

        QUnit.ok(!test.productUiVariationattributeUnity('isEmpty'), 'isEmpty is working when prefilled');
        QUnit.ok(test.productUiVariationattributeUnity('isValid'), 'isValid is working when prefilled');
        input.val('test1').trigger('input');

        setTimeout(function() {
            QUnit.equal(list.children().eq(2).find('input').val(), 'test1', 'first additional input has "test1" as value');
            list.children().eq(2).find('input').val('').trigger('input');

            setTimeout(function() {
                QUnit.ok(!test.productUiVariationattributeUnity('isEmpty'), 'isEmpty is working when prefilled');
                QUnit.ok(test.productUiVariationattributeUnity('isValid'), 'isValid is working when prefilled');

                fixture.destroy();
                QUnit.start();
            }, 500);
        }, 500);
    });

    QUnit.asyncTest("test validation", function () {
        fixture.init();
        var test = $('#test1'),
            input = test.find('.ep-variationassistant-attributevalueinput'),
            list = $('#test1 .ep-variationassistant-attributelist');

        $('.ep-js').metaparse();

        test.productUiVariationattributeUnity(options);

        QUnit.ok(test.productUiVariationattributeUnity('isEmpty'), 'isEmpty is working when empty');
        QUnit.ok(!test.productUiVariationattributeUnity('isValid'), 'isValid is working when empty');
        list.children().eq(0).find('input[type="text"]').val('test1').trigger('input');

        setTimeout(function() {
            list.children().eq(0).find('input[type="text"]').val('').trigger('input');

            setTimeout(function() {
                QUnit.ok(!list.children().eq(0).find('input[type="text"]').data('valid'), 'first input shows error if filled and emptied again');
                QUnit.ok(list.children().eq(0).find('input[type="text"]').hasClass('ui-invalid'), 'first input has errorclass if filled and emptied again');
                list.children().eq(0).find('input[type="text"]').val('test').trigger('input');

                setTimeout(function() {
                    QUnit.ok(list.children().eq(0).find('input[type="text"]').data('valid'), 'first input no longer shows error if refilled');
                    QUnit.ok(!list.children().eq(0).find('input[type="text"]').hasClass('ui-invalid'), 'first input no longer has errorclass if refilled');

                    input.val('test').trigger('input');
                    setTimeout(function() {
                        input.val('test').trigger('input');
                        setTimeout(function() {
                            input.val('test').trigger('input');
                            setTimeout(function() {
                                input.trigger('focus');
                                QUnit.ok(list.children().find('input[type="text"]').get().every(function(el) {return (el.value === 'test'); }), 'all values are "test"');
                                QUnit.equal(list.children().find('input[type="text"]').length, 4, '4 inputs exist');
                                QUnit.equal(list.children().find('input[type="text"]').get().filter(function(el) {return $(el).hasClass('ui-invalid');}).length, 4, 'all elements have ui-invalid class');
                                QUnit.equal(list.children().find('input[type="text"]').get().filter(function(el) {return !$(el).data('valid');}).length, 4, 'all elements valid data is invalid');
                                list.children().eq(0).find('input[type="text"]').val('test1').trigger('input');

                                setTimeout(function() {
                                    input.trigger('focus');
                                    QUnit.equal(list.children().find('input[type="text"]').get().filter(function(el) {return (el.value === 'test');}).length, 3, '3 values are "test"');
                                    QUnit.equal(list.children().find('input[type="text"]').length, 4, '4 inputs exist');
                                    QUnit.equal(list.children().find('input[type="text"]').get().filter(function(el) {return $(el).hasClass('ui-invalid');}).length, 3, '3 elements have ui-invalid class');
                                    QUnit.equal(list.children().find('input[type="text"]').get().filter(function(el) {return !$(el).data('valid');}).length, 3, '3 elements valid data is invalid');
                                    QUnit.ok(list.children().eq(0).find('input[type="text"]').data('valid'), 'first element valid data is valid');
                                    QUnit.ok(!list.children().eq(0).find('input[type="text"]').hasClass('ui-invalid'), 'first element doesnt have invalid class');

                                    list.children().eq(1).find('input[type="text"]').val('test2').trigger('input');
                                    list.children().eq(2).find('input[type="text"]').val('test3').trigger('input');

                                    setTimeout(function() {
                                        input.trigger('focus');
                                        QUnit.equal(list.children().find('input[type="text"]').get().filter(function(el) {return (el.value === 'test');}).length, 1, '1 value is "test"');
                                        QUnit.equal(list.children().find('input[type="text"]').length, 4, '4 inputs exist');
                                        QUnit.equal(list.children().find('input[type="text"]').get().filter(function(el) {return $(el).hasClass('ui-invalid');}).length, 0, '0 elements have ui-invalid class');
                                        QUnit.equal(list.children().find('input[type="text"]').get().filter(function(el) {return !$(el).data('valid');}).length, 0, '0 elements valid data is invalid');
                                        QUnit.ok(test.productUiVariationattributeUnity('isValid'), 'isValid is working when not empty');
                                        QUnit.ok(!test.productUiVariationattributeUnity('isEmpty'), 'isEmpty is working when full');

                                        fixture.destroy();
                                        QUnit.start();
                                    }, 1000);
                                }, 1000);
                            }, 1000);
                        }, 300);
                    }, 300);
                }, 1000);
            }, 1000);
        }, 1000);
    });

    QUnit.asyncTest("test basic usage", function () {
        fixture.init();
        var test = $('#test1'),
            input = test.find('.ep-variationassistant-attributevalueinput'),
            list = $('#test1 .ep-variationassistant-attributelist');


        test.productUiVariationattributeUnity(options);

        QUnit.equal(list.children().length, 1, 'one input is available');

        input.val('test1').trigger('input');

        setTimeout(function() {
            QUnit.equal(input.val(), 'test1', 'string in input is "test1"');
            QUnit.equal(list.children().length, 1, 'no second input is created after 100ms');

            setTimeout(function() {
                QUnit.equal(input.val(), '', 'input is empty');
                QUnit.equal(list.children().length, 2, 'second input is created after 300ms');
                QUnit.equal(list.children().eq(1).find('input').val(), 'test1', 'string in 2nd row input is "test"');

                input.val('test2').trigger('input');

                setTimeout(function() {
                    QUnit.equal(input.val(), 'test2', 'string in input is "test"');
                    QUnit.equal(list.children().length, 2, 'no third input is created after 100ms');

                    input.val('test3').trigger('input');

                    setTimeout(function() {
                        QUnit.equal(input.val(), 'test3', 'string in input is "test"');
                        QUnit.equal(list.children().length, 2, 'no third input is created after 250ms');

                        setTimeout(function() {
                            QUnit.equal(input.val(), '', 'input is empty');
                            QUnit.equal(list.children().length, 3, 'third input is created after 300ms');
                            QUnit.equal(list.children().eq(2).find('input').val(), 'test3', 'string in third row input is "test"');

                            list.children().eq(1).find('i').click();

                            setTimeout(function() {
                                QUnit.equal(list.children().length, 2, 'row was deleted');

                                input.val('test4').trigger('input');
                                setTimeout(function() {
                                    input.val('test5').trigger('input');
                                    setTimeout(function() {
                                        input.val('test6').trigger('input');
                                        setTimeout(function() {
                                            input.val('test7').trigger('input');
                                            setTimeout(function() {
                                                input.val('test8').trigger('input');
                                                setTimeout(function() {
                                                    input.val('test9').trigger('input');
                                                    setTimeout(function() {
                                                        input.val('test2').trigger('input');
                                                        setTimeout(function() {
                                                            input.val('test1').trigger('input');
                                                            QUnit.ok(list.children().eq(0).find('i').hasClass('HideElement'), 'initial input delete icon hidden before input');
                                                            list.children().eq(0).find('input').val('test0').trigger('input');

                                                            setTimeout(function() {
                                                                QUnit.equal(list.children().length, 10, '10 rows created');
                                                                QUnit.ok(input.closest('.row').hasClass('HideElement'), 'input hidden after 10. row');
                                                                QUnit.ok(!list.children().eq(0).find('i').hasClass('HideElement'), 'initial input delete icon no longer hidden after input');
                                                                list.children().eq(1).find('i').click();

                                                                setTimeout(function() {
                                                                    QUnit.equal(list.children().length, 9, ' 2nd row was deleted');
                                                                    QUnit.ok(!input.closest('.row').hasClass('HideElement'), 'input visible again after deleting 2nd row');

                                                                    values = Array.apply(null, {length: 10}).map(function(key, index) {return 'test' + index;});
                                                                    QUnit.ok(test.productUiVariationattributeUnity('getSelectedValues').every(function(val){return (values.indexOf(val) !== -1);}), 'getValues is working');


                                                                    fixture.destroy();
                                                                    QUnit.start();
                                                                }, 20);
                                                            }, 300);
                                                        }, 300);
                                                    }, 300);
                                                }, 300);
                                            }, 300);
                                        }, 300);
                                    }, 300);
                                }, 300);
                            }, 10);
                        }, 100);
                    }, 150);
                }, 100);
            }, 200);
        }, 100);
    });
});